#!/usr/bin/env python3
"""
Packet Analyzer
Deep packet inspection and analysis tool.
Parameters:
    - interface: Network interface to capture packets
    - filter: BPF filter for packet capture
    - output: Output file for analysis results
Requires: scapy package and admin/root privileges
"""
from scapy.all import *
import argparse
import logging
import json
from datetime import datetime
import collections
from pathlib import Path
import threading
import queue

class PacketAnalyzer:
    def __init__(self, interface, packet_filter=None, output=None):
        self.interface = interface
        self.packet_filter = packet_filter
        self.output_file = output
        self.packet_queue = queue.Queue()
        self.stop_event = threading.Event()
        self.stats = {
            'total_packets': 0,
            'protocols': collections.Counter(),
            'packet_sizes': collections.Counter(),
            'conversations': collections.defaultdict(int),
            'tcp_flags': collections.Counter(),
            'dns_queries': collections.Counter(),
            'http_methods': collections.Counter()
        }
        self.setup_logging()

    def setup_logging(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger(__name__)

    def analyze_tcp_flags(self, packet):
        if TCP in packet:
            flags = []
            if packet[TCP].flags.S: flags.append('SYN')
            if packet[TCP].flags.A: flags.append('ACK')
            if packet[TCP].flags.F: flags.append('FIN')
            if packet[TCP].flags.R: flags.append('RST')
            if packet[TCP].flags.P: flags.append('PSH')
            if packet[TCP].flags.U: flags.append('URG')
            
            flag_combo = '|'.join(flags)
            if flag_combo:
                self.stats['tcp_flags'][flag_combo] += 1

    def analyze_dns(self, packet):
        if DNS in packet and packet.haslayer(DNSQR):
            query = packet[DNSQR].qname.decode()
            self.stats['dns_queries'][query] += 1

    def analyze_http(self, packet):
        if TCP in packet and packet[TCP].dport == 80:
            payload = str(packet[TCP].payload)
            for method in ['GET', 'POST', 'PUT', 'DELETE']:
                if method in payload:
                    self.stats['http_methods'][method] += 1

    def packet_callback(self, packet):
        self.packet_queue.put(packet)
        self.stats['total_packets'] += 1
        self.stats['packet_sizes'][len(packet)] += 1

        if IP in packet:
            conversation = f"{packet[IP].src} -> {packet[IP].dst}"
            self.stats['conversations'][conversation] += 1

            if TCP in packet:
                self.stats['protocols']['TCP'] += 1
                self.analyze_tcp_flags(packet)
            elif UDP in packet:
                self.stats['protocols']['UDP'] += 1
            elif ICMP in packet:
                self.stats['protocols']['ICMP'] += 1

            if DNS in packet:
                self.analyze_dns(packet)
            
            self.analyze_http(packet)

    def capture_packets(self):
        try:
            self.logger.info(f"Starting packet capture on interface {self.interface}")
            sniff(iface=self.interface, filter=self.packet_filter,
                  prn=self.packet_callback, store=0,
                  stop_filter=lambda _: self.stop_event.is_set())
        except Exception as e:
            self.logger.error(f"Error during packet capture: {str(e)}")

    def save_results(self):
        if self.output_file:
            try:
                results = {
                    'timestamp': datetime.now().isoformat(),
                    'interface': self.interface,
                    'filter': self.packet_filter,
                    'statistics': {
                        'total_packets': self.stats['total_packets'],
                        'protocols': dict(self.stats['protocols'].most_common(10)),
                        'top_conversations': dict(sorted(
                            self.stats['conversations'].items(),
                            key=lambda x: x[1],
                            reverse=True
                        )[:10]),
                        'tcp_flags': dict(self.stats['tcp_flags'].most_common(10)),
                        'packet_size_distribution': dict(
                            self.stats['packet_sizes'].most_common(10)
                        ),
                        'dns_queries': dict(self.stats['dns_queries'].most_common(10)),
                        'http_methods': dict(self.stats['http_methods'])
                    }
                }
                
                with open(self.output_file, 'w') as f:
                    json.dump(results, f, indent=4)
                self.logger.info(f"Analysis results saved to {self.output_file}")
            except Exception as e:
                self.logger.error(f"Error saving results: {str(e)}")

    def start_analysis(self):
        capture_thread = threading.Thread(target=self.capture_packets)
        capture_thread.start()
        
        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            self.logger.info("Stopping packet analysis...")
            self.stop_event.set()
            capture_thread.join()
            self.save_results()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Packet Analyzer")
    parser.add_argument("interface", help="Network interface to capture packets")
    parser.add_argument("--filter", help="BPF filter for packet capture")
    parser.add_argument("--output", help="Output file for analysis results")
    args = parser.parse_args()

    analyzer = PacketAnalyzer(args.interface, args.filter, args.output)
    analyzer.start_analysis()
